/***************************************************************************
* Titel        : CT04.C
*
* Description  : Einfaches Multitasking fr den MSP430
*
* Environment  :
*
* Responsible  : J. Altenburg
*
* Guidelines   :
*
* Revision List:
  --------------------------------------------------------------------------
  Date     | Author          | Change
  --------------------------------------------------------------------------
  13.01.04 |J. Altenburg     | Ersterstellung
  --------------------------------------------------------------------------
           |                 |
  --------------------------------------------------------------------------
****************************************************************************/

/***************************************************************************
* HEADER-FILES (Only those that are needed in this file)                   *
****************************************************************************/
/* System-headerfiles */
#include "cdef.h"                      /* General definitions and types */

#include "msp430x14x.h"
#include "msp430def.h"

/* Own headerfiles */


/* Foreign headerfiles */
#include "uexec.h"

/***************************************************************************
* Defines und Makros Zugriff                                               *
****************************************************************************/

/* Prototyp fr Funktionspointer */
typedef void (* fLocalFunc)( void ); /* Funktionspointer */



/***************************************************************************
*    Variablen                                                             *
****************************************************************************/
struct stTimer{
     word Run  : 1; /* Scheduling erlauben */
     word F1   : 1; /* Timer-Flags, z.B. fr Zhlaufgaben */
     } stTimer;

word wTaskId0; /*
* Description : Task-ID
*/

word wTaskId1; /*
* Description : Task-ID
*/

word wTaskId2;

word wRegister; /*
* Description : Hilfsregister
*/

word w0;
word w1;

byte bJA;

/***************************************************************************
*   Stackbereiche fr Multitasking
****************************************************************************/
byte abTask0[UEXC_MIN_STACK_SIZE]; /*
* Description : Stackbereich fr Task 0
*/

byte abTask1[UEXC_MIN_STACK_SIZE]; /*
* Description : Stackbereich fr Task 1
*/


byte abTask2[UEXC_MIN_STACK_SIZE]; /*
* Description : Stackbereich fr Task 1
*/

/***************************************************************************
*    Beispieltasks
****************************************************************************/
void vTask2( void ){
	 word w;
	 byte i;
	 for(i = 0; i < 8; i++){
	 	   w = 30000;
		   while(w--);
           P2OUT ^= 0x04;	   /* Port P2.3 toggelt */
		   }
	 }

void vTask0( void ){
	 word w;
	 while(1){
	      w = 60000;
		  while(w--);
		  P2OUT ^= 0x01;	   /* Port P2.0 togglet */
		  }
     }

void vTask1( void ){
	 dword dw;
	 while(1){
	      dw = 80000;
		  while(dw--);
		  P2OUT ^= 0x02; 	   /* Port 2.1 togglet */
          wTaskId2 = UEXC_CreateTask(vTask2, &abTask2[0], sizeof (abTask2), 1);	   
		  }
     }


	   

/***************************************************************************
*    Funktion: Hardware-Initialisierung (Application Builder)              *
*                                                                          *
*    Autor:    Jens Altenburg                                              *
****************************************************************************/
void vClockInit( void ){
     word w;
     // DCOCTL  = 0x00;
     WDTCTL = WDTPW + WDTHOLD;     /* Watchdog deaktivieren */
#if 1
     P5DIR |= 0x10;                /* 8MHz-CLK am Pin 48 */
     P5SEL |= 0x10;
     P5DIR |= 0x20;                /* SMCLK an Pin 49 */
     P5SEL |= 0x20;
#endif
     BCSCTL1 &= ~XT2OFF;           /* Oszillator on */
     do  {
          IFG1 &= ~OFIFG;
          for(w = 0xff; w > 0; w--);
          }
     while((IFG1 & OFIFG) != 0);
     BCSCTL2 = 0x88;
//     BCSCTL2 |= SELM1 + SELS + DIVS0;   /* nun los */
     }

void vPortInit( void ){
     P1OUT   = 0x00;               /* Schrittmotor 1 */
     P1DIR   = 0x7F;
     P1IES   = 0x00;
     P1IE    = 0x00;
     P1SEL   = 0x00;

     P2OUT   = 0xFF;               /* LEDs */
     P2DIR   = 0xFF;
     P2IES   = 0x00;
     P2IE    = 0x00;
     P2SEL   = 0x00;

     P3OUT   = 0x00;               /* UART0, P3.4 output */
     P3DIR   = 0x10;
     P3SEL   = 0x30;

     P4OUT   = 0xFE;               /* CFG - Port */
     P4DIR   = 0x7f;
     P4SEL   = 0x00;
#if 0
     P5OUT   = 0x00;               /* not used */
     P5DIR   = 0x00;
     P5SEL   = 0x00;
#endif
     P6OUT   = 0x00;               /* not used */
     P6DIR   = 0x00;
     P6SEL   = 0x03;                  /* A/D-Wandler */
     }


void vTimeraInit( void ){
     TACTL = TASSEL1 + TACLR;      /* SMCLK, clear TAR */
     CCTL0 = CCIE;                 /* CCR0 interrupt enable */
     CCR0 = 10000;                 /* 2,5 ms Timer Click */
     TACTL |= MC0;                 /* Up-Mode */
     }


//call this routine to initialise all peripherals
void vInitDevices( void ){
     DINT();                       /* disable all interrupts */
//   ME1     = 0x00;               /* disable sfr peripherals */
//   ME2     = 0x00;               /* disable sfr peripherals */
//   IE1     = 0x00;               /* disable sfr interrupts */
//   IE2     = 0x00;               /* disable sfr interrupts */

//   WDTCTL  = 0x5A | 0x15;        /* watchdog initialisation including nmi function */

     vClockInit();                 /* initialise other peripherals */
     vPortInit();
//     vTimeraInit();			   /* erledigt UEXEC */

//   ME1     = 0xC0;               /* enable sfr peripherals */
//   ME2     = 0x00;               /* enable sfr peripherals */
//   IE1     = 0xC0;               /* enable sfr interrupts */
//   IE2     = 0x03;               /* disable sfr interrupts */
//   EINT();                       /* re-enable interrupts */
     }



/***************************************************************************
*    Main-Loop                                                             *
*    - initialisiert Module und Variable                                   *
*    - die stndig gerufenen Funktionen stehen in der Auftargsliste        *
*
****************************************************************************/
int main(void){
     vInitDevices();
     wTaskId0 = UEXC_CreateTask(vTask0, &abTask0[0], sizeof (abTask0), 1);
     wTaskId1 = UEXC_CreateTask(vTask1, &abTask1[0], sizeof (abTask1), 1);
     UEXC_StartScheduler();
     while (1);                         /* wird nie erreicht */
     return 1;                          /* unterdrckt Compilerwarnung */
     }

/***************************************************************************
* Kurzbeschreibung     : der Timer kontrolliert den zyklischen Aufruf
*                        des Schedulers
* Author               : Jens Altenburg
****************************************************************************/
#pragma interrupt_handler timera_0_isr:TIMERA0_VECTOR
void timera_0_isr(void){
      CCR0 = 20000;                     /* 2,5 ms Takt */
	  P2OUT ^= 0x80;                    /* zum Taktmessen */
asm(
	" push  r4\n"
	" push  r5\n"
	" push  r6\n"
	" push  r7\n"
	" push  r8\n"
	" push  r9\n"
	" push  r10\n"
	" push  r11\n"
	" push  r12\n"
	" push  r13\n"
	" push  r14\n"
	" push  r15\n"
	" mov 	SP,%uexc_current_sp\n" 		/* Stackpointer sichern */
   	" call	#%UEXC_CheckTask\n"			/* Task-State prfen */
	" pop   r15\n"
	" pop   r14\n"
	" pop   r13\n"
	" pop   r12\n"
	" pop   r11\n"
	" pop   r10\n"
	" pop   r9\n"
	" pop   r8\n"
	" pop   r7\n"
	" pop   r6\n"
	" pop   r5\n"
	" pop   r4\n"
	" reti\n");
	}

/***************************************************************************
* Kurzbeschreibung     : "symbolischer" Timerstart
* Author               : Jens Altenburg
****************************************************************************/
void UEXC_StartTimer(void){
     vTimeraInit();
     EINT();	   /* Scheduler enables interrupts */
	 }


/***************************************************************************
* Kurzbeschreibung     : save state and then reschedule
*                        this called by Defer to give up control
* Author               : Jens Altenburg
****************************************************************************/
void UEXC_SavregsAndResched(void){
asm(
	" mov     SR,%wRegister\n"           /* interrupt stack frame */
	" bis     #8,%wRegister\n"		 	 /* enable GIE */
	" push.w  %wRegister\n"				 /* "SR" to stack */
	" push  r4\n"
	" push  r5\n"
	" push  r6\n"
	" push  r7\n"
	" push  r8\n"
	" push  r9\n"
	" push  r10\n"
	" push  r11\n"
	" push  r12\n"
	" push  r13\n"
	" push  r14\n"
	" push  r15\n"
	" mov.w   r15,%wRegister\n"			 /* r15 for indexed adressing */
	" mov.w   %uexc_current_sp,r15\n"
	" mov.w	  SP,(r15)\n"	 		 	 /* save stackpointer */
	" mov.w	  %wRegister,r15\n"
	" br	  #%UEXC_Schedule\n"		 /* Schedule */
	);
    }


/***************************************************************************
* Kurzbeschreibung     : Start a New Task
* Author               : Jens Altenburg
****************************************************************************/
void UEXC_StartNewTask(void){
asm(
	" mov     %uexc_current_sp, SP\n"	 /* moves content of uexc_cu.. to SP */
	" push.w  #%UEXC_KillSelf\n" 		 /* Killself() onto stack */
	" push.w  %uexc_current_func\n"	 	 /* new function to stack */
	" mov	  SR,%wRegister\n"		 	 /* Statusregister bearbeiten */
	" bis	  #8,%wRegister\n"		 	 /* GIE */
	" push.w  %wRegister\n"			 	 /* statusregister to stack */
	" reti\n"						 	 /* go */
	);	 
    }

/***************************************************************************
* Kurzbeschreibung     : Resume a previously stopped task
* Uebergabeparameter   :
* Return Value         :
* Author               : Jens Altenburg
****************************************************************************/
void UEXC_Resume(void){
asm(
	" mov     %uexc_current_sp, SP\n"    /* Stackpointer setzen */
	" pop   r15\n"
	" pop   r14\n"
	" pop   r13\n"
	" pop   r12\n"
	" pop   r11\n"
	" pop   r10\n"
	" pop   r9\n"
	" pop   r8\n"
	" pop   r7\n"
	" pop   r6\n"
	" pop   r5\n"
	" pop   r4\n"
	" reti\n"				  		   	 /* ... */
	);
    }


/***************************************************************************
* EOF: CT04.C
****************************************************************************/

