/***************************************************************************
* Titel        : icc430.C
*
* Description  : Portierung UEXEC auf MSP430
*
* Environment  :
*
* Responsible  : J. Altenburg
*
* Guidelines   :
*
* Revision List:
  --------------------------------------------------------------------------
  Date     | Author          | Change
  --------------------------------------------------------------------------
  27.01.04 |J. Altenburg     | Ersterstellung
  --------------------------------------------------------------------------
           |                 |
  --------------------------------------------------------------------------
****************************************************************************/
#include "cdef.h"
#include "_uexec.h"

const char abCopyright1[] = "MSP430 Multitasking System";
const char abCopyright2[] = "(c) 2004 J. Altenburg";


/***************************************************************************
* Kurzbeschreibung     : Timer A3 kontrolliert den zyklischen Aufruf des Schedulers
* Uebergabeparameter   : keiner
* Return Value         : keiner
* Author               : Jens Altenburg
****************************************************************************/
void far vTimerA3ISR(void);
#pragma INTERRUPT vTimerA3ISR

void far vTimerA3ISR(void)
    {
    if(stSystem.Task == 1)                  /* Taskwechsel erlaubt ? */
        {
#if 0
    ; /* check if we had interrupted the monitor */
        MOV.B       17[SP], R0H             ; /* get PC(H) */
        AND.B       R0H, 0FH                ; /* mask flags out of PC(H) */
        MOV.B       15[SP], R0L             ; /* get PC(M) */
        CMP.W       #0fc0H, R0              ; /* monitor interrupted ? */
        JLTU        ?+
        POPM        R0,R1,R2,R3,A0,A1,FB
        REIT                                ; /* ==>> yes (therefore: do NOT switch task) */
        NOP
?:                                          ; /* ==>> no */
        /* save stack pointer */
        STC         SP, _uexc_current_sp    ; /* save stackpointer */
        JSR.a       _UEXC_CheckTask         ; /* check task state */
        POPM        R0,R1,R2,R3,A0,A1,FB
        REIT                                ; /* ==>> exit */
        NOP
#endif
        }
    }



/***************************************************************************
* Kurzbeschreibung     : Initialisierung Timer A3
* Uebergabeparameter   :
* Return Value         :
* Author               : Jens Altenburg
****************************************************************************/
void UEXC_StartTimer(void)
    {
#if 0
    TA3MR = 0x40;                   /* Timer mode: timer mode, f/8 */
    /* 2000 Hz = 10 MHz / 8) /  625  */
    /* 2000 Hz = 16 MHz / 8) / 1000  */
    /*  100 Hz = 16 MHz / 8) / 20000 */
    TA3 = (unsigned short)(nOsTick - 1);
    mcDisInt();
    TA3IC = enLevel1;               /* Timer A3 interrupt control register; Level 1 */
    mcEnaInt();
    TA3S  = 1;                      /* startet Timer A3 */
#endif
    }


/***************************************************************************
* Kurzbeschreibung     : save state and then reschedule
*                        this called by Defer to give up control
* Uebergabeparameter   :
* Return Value         :
* Author               : Jens Altenburg
****************************************************************************/
void UEXC_SavregsAndResched(void)
    {
#if 0
;   /* first create an interrupt stack frame */
;   /* PC_l, PC_m, PC_h vom Stack holen */
    POP.w   R2                      ; /* PC_l und PC_m */
    POP.b   R1L                     ; /* PC_h laden */
    AND.b   #0fh,R1L
    STC     FLG,R0                  ; /* FLG holen */
    AND.w   #00ffh,R0               ; /* 0000000011111111 Maske aufsetzen */
    OR.w    #0040h,R0               ; /* re-enable interrupt after REIT */
    OR.b    R1L,R0H                 ; /* PC_h aufodern */
    PUSH.w  R0                      ; /* FLG_h + PC_h */
    PUSH.w  R2                      ; /* PC_m, PC_l */
    PUSHM   R0,R1,R2,R3,A0,A1,FB
;   /* now save stack pointer */
    MOV.w   _uexc_current_sp,A0
    STC     SP,[A0]                 ; /* now store the stack pointer to *_current_sp, which is set to &_current_task->sp */
    JMP.a   _UEXC_Schedule          ; /* now reschedule */
#endif
    }


/***************************************************************************
* Kurzbeschreibung     : Start a new task
* Uebergabeparameter   :
* Return Value         :
* Author               : Jens Altenburg
****************************************************************************/
void UEXC_StartNewTask(void)
    {
#if 0
    LDC     _uexc_current_sp,SP     ; /* load sp with new task address */
;   /* make KillSelf as the "return pc" of a new task, so if it ever returns, it will reclaim the storage */
    PUSH.b  #(_UEXC_KillSelf >> 16) ;
    PUSH.w  #(_UEXC_KillSelf & 0ffffh)
;    /* new task stack frame:  */
;    /* FLG_h, PC_h            */
;    /* FLG,PC_m  PC_l         */
;    /* puzzle FLG and PC togehter */
    STC     FLG,R0                  ; /* FLG holen */
    AND.w   #00ffh,R0               ; /* 0000000011111111 Maske aufsetzen */
    OR.w    #0040h,R0               ; /* re-enable interrupt after REIT */
    OR.b    (_uexc_current_func + 2),R0H
;    /* create stack frame */
    PUSH.w  R0                      ; /* FLG_h + PC_h */
    PUSH.w  (_uexc_current_func & 0ffffh)
    REIT                            ; /* return from interrupt */
    NOP
#endif
    }

/***************************************************************************
* Kurzbeschreibung     : Resume a previously stopped task
* Uebergabeparameter   :
* Return Value         :
* Author               : Jens Altenburg
****************************************************************************/
void UEXC_Resume(void)
    {
#pragma ASM
; /* _current_sp has current_task->sp restore the stack pointer */
; /* and just pretend we are returning from interrupt */
    LDC     _uexc_current_sp, SP
    POPM    R0,R1,R2,R3,A0,A1,FB
    REIT                            ; /* REIT because of call of function during interrupt */
    NOP
#pragma ENDASM
    }


/***************************************************************************

/* EOF system.c */

